
library("votesmart")

# Put in API key to access data
Sys.setenv(VOTESMART_API_KEY = "")

##### Candidates #####

# pull in candidate information
state_info <- read.csv("Data/senate_race_info18.csv", stringsAsFactors = F)[,-9] # remove notes column

# vector of Senate candidate names
candidates <- c(state_info$dem_cand, state_info$rep_cand)
candidates

# vector of names of same-state/same-party senators
samestate <- c(
  "Kamala Harris",
  "Richard Blumenthal",
  "Brian Schatz",
  "Ed Markey",
  "Chris Van Hollen",
  "Gary Peters",
  "Mike Lee",
  "Tina Smith",
  "Cindy Hyde-Smith",
  "Ben Sasse",
  "Cory Booker",
  "Tom Udall",
  "Chuck Schumer",
  "Lamar Alexander",
  "John Cornyn",
  "Marco Rubio",
  "Todd Young",
  "Roy Blunt",
  "Catherine Cortez Masto",
  "John Hoeven",
  "Mark Warner",
  "Patty Murray",
  "Mike Enzi"
)

# vector of winning candidates
candidates_win <- c()
for (i in 1:nrow(state_info)){
  if (state_info$winner[i] == "dem"){
    candidates_win[i] <- state_info$dem_cand[i]
  }else if (state_info$winner[i] == "rep"){
    candidates_win[i] <- state_info$rep_cand[i]
  }else{
    candidates_win[i] <- "Angus King"
  }
}
candidates_win

# last names
lastnames <- sapply(strsplit(candidates_win, " "), tail, 1)
lastnames[lastnames == "Jr."] <- "Casey"
lastnames[lastnames == "Barasso"] <- "Barrasso"

# first names 
firstnames <- sapply(strsplit(candidates_win, " "), `[`, 1)
firstnames[lastnames == "Menendez"] <- "Robert"
firstnames[lastnames == "Gillibrand"] <- "Kirsten"
firstnames[lastnames == "Jr."] <- "Robert"
firstnames[lastnames == "Kaine"] <- "Timothy"
firstnames[lastnames == "Manchin"] <- "Joseph"

# first names same-state
firstnames_ss <- sapply(strsplit(samestate, " "), `[`, 1)

# get candidate attributes from votesmart
cand_attr <- candidates_get_by_lastname(lastnames, 2018, "G")

# find matches
cand_attr$match <- ifelse((cand_attr$first_name %in% firstnames & cand_attr$last_name %in% lastnames) | 
                            cand_attr$ballot_name %in% candidates_win, 1, 0)
cand_attr$ballot_name[cand_attr$match == 1]

# how many duplicate last names?
table(cand_attr$last_name[cand_attr$match == 1])

# remove erroneous matches
cand_attr$match[cand_attr$ballot_name == "Rick Braun"] <-  0 # mike braun is correct
cand_attr$match[cand_attr$ballot_name == "Jon S. Cardin"] <-  0 # ben cardin is correct
cand_attr$match[cand_attr$ballot_name == "John Heinrich"] <-  0 # martin heinrich is correct
cand_attr$match[cand_attr$last_name == "King" & cand_attr$first_name != "Angus"] <-  0 # angus king is correct
cand_attr$match[cand_attr$last_name == "Cruz" & cand_attr$first_name != "Rafael"] <-  0 # ted cruz is correct
cand_attr$match[cand_attr$last_name == "Brown" & cand_attr$ballot_name != "Sherrod Brown"] <-  0 # sherrod brown is correct
cand_attr$match[cand_attr$last_name == "Scott" & cand_attr$first_name != "Richard"] <-  0 # richard (rick) scott is correct
cand_attr$match[cand_attr$last_name == "Warren" & cand_attr$ballot_name != "Elizabeth A. Warren"] <-  0 # elizabeth a. warren is correct
cand_attr$match[cand_attr$last_name == "Fischer" & cand_attr$ballot_name != "Deb Fischer"] <-  0 # deb fischer is correct
cand_attr$match[cand_attr$last_name == "Murphy" & cand_attr$ballot_name != "Chris Murphy"] <-  0 # chris murphy is correct
cand_attr$match[cand_attr$ballot_name == "Chris Murphy" & cand_attr$title != "Senator"] <-  0 # remove additional mismatch
cand_attr$match[cand_attr$last_name == "Sanders" & cand_attr$first_name != "Bernard"] <-  0 # bernie sanders is correct
cand_attr$match[cand_attr$last_name == "Wicker" & cand_attr$ballot_name != "Roger F. Wicker"] <-  0 # chris murphy is correct

# no remaining duplicates
table(cand_attr$last_name[cand_attr$match == 1])

# no missing names
lastnames %in% unique(cand_attr$last_name[cand_attr$match==1])

# keep only matching names
cand_attr <- subset(cand_attr, match == 1)

##### Retrieve interest group ratings #####

# list of relevant special interest groups and their codes in vote smart
interest_group <- c(
  252, # NRLC
  1734, # The Club for Growth
  1256, # Human Rights Campaign
  893, # National Association for the Advancement of Colored People (NAACP)
  599, # NORML
  2622, # AFA Action
  1012, # League of Conservation Voters
  1034, # National Rifle Association
  1985, # Numbers USA
  1161, # American Federation of Labor and Congress of Industrial Organizations (AFL-CIO)
  1744 # RESULTS
  )

# scrape ratings
ratings <- rating_get_candidate_ratings(
  candidate_ids = cand_attr$candidate_id,
  sig_ids = interest_group
  )

# change ratings to numeric
ratings$rating <- as.numeric(ratings$rating)

# check data
nrow(ratings)
table(ratings$timespan)

# get rid of lifetime ratings
ratings <- subset(ratings, rating_name %in% c("Positions","Positions [10-12-2020]"))
table(ratings$rating_name)

# keep only ratings in 2019-2020, except for HRC which is 2017-2020
ratings <- subset(ratings, timespan %in% c("2019","2019-2020","2020","2017-2020"))
table(ratings$timespan)

# determine time range for each SIG
sig_range <- list()
for (i in 1:length(interest_group)){
  sig_range[[i]] <- interest_group[i]
  sig_range[[i]] <- unique(ratings$timespan[ratings$sig_id == interest_group[i]])
}
names(sig_range) <- interest_group

# get rid of redundant ratings
rows <- list()
for (i in 1:length(interest_group)){
  if ("2019-2020" %in% sig_range[[i]]){
    rows[[i]] <- which(ratings$sig_id == interest_group[[i]] & ratings$timespan != "2019-2020")
  }
}
ratings <- ratings[-unlist(rows),]

# determine time range for each SIG
sig_range <- list()
for (i in 1:length(interest_group)){
  sig_range[[i]] <- interest_group[i]
  sig_range[[i]] <- unique(ratings$timespan[ratings$sig_id == interest_group[i]])
}
names(sig_range) <- interest_group
sig_range

# SIGs with multiple ratings in relevant period
multiples <- c()
for (i in 1:length(sig_range)){
  if (length(sig_range[[i]]) > 1){multiples[i] <- names(sig_range)[i]}
}
multiples <- multiples[!is.na(multiples)]
multiples

# average for SIGs w/multiple ratings
temp <- ratings[which(ratings$sig_id %in% multiples),]
temp <- aggregate(temp, 
          by=list(temp$candidate_id, temp$sig_id), 
          mean, na.rm = T)
ratings$rating_avg <- ratings$rating
for (j in 1:length(multiples)){
  for (i in 1:nrow(ratings)){
    if (ratings[i,"sig_id"] == multiples[j]){
      ratings$rating_avg[i] <- temp$rating[temp$Group.1 == ratings$candidate_id[i] & temp$Group.2 == multiples[j]] 
    }
  }
}

# how many/which available interest groups?
unique(ratings$sig_id)

# candidate ballot names and states and parties
cand_data <- cbind(cand_attr$candidate_id, cand_attr$ballot_name, cand_attr$election_state_id, cand_attr$office_parties)

# export
write.csv(ratings, "Data/Vote Smart Ratings.csv", row.names = F)
write.csv(cand_data, "Data/Candidates.csv", row.names = F)



##### Get all Senators ratings during 2018-2020 term #####

# list of relevant special interest groups and their codes in vote smart
interest_group <- c(
  252, # NRLC
  1734, # The Club for Growth
  1256, # Human Rights Campaign
  893, # National Association for the Advancement of Colored People (NAACP)
  599, # NORML
  2622, # AFA Action
  1012, # League of Conservation Voters
  1034, # National Rifle Association
  1985, # Numbers USA
  1161, # American Federation of Labor and Congress of Industrial Organizations (AFL-CIO)
  1744 # RESULTS
)

# retrive IDs for all Senators
all_ids <- candidates_get_by_office_state(state_ids = state.abb, office_ids = 6, election_years = c(2014,2016,2018))
all_ids <- subset(all_ids, election_status == "Won")
all_ids <- as.data.frame(all_ids[,c("candidate_id","ballot_name","election_parties","election_status")])
all_ids

# scrape ratings
ratings <- rating_get_candidate_ratings(
  candidate_ids = all_ids$candidate_id,
  sig_ids = interest_group
)

# change ratings to numeric
ratings$rating <- as.numeric(ratings$rating)

# check data
nrow(ratings)
table(ratings$timespan)

# get rid of lifetime ratings
ratings <- subset(ratings, rating_name %in% c("Positions","Positions [10-12-2020]"))
table(ratings$rating_name)

# keep only ratings in 2019-2020, except for HRC which is 2017-2020
ratings <- subset(ratings, timespan %in% c("2019","2019-2020","2020","2017-2020"))
table(ratings$timespan)

# determine time range for each SIG
sig_range <- list()
for (i in 1:length(interest_group)){
  sig_range[[i]] <- interest_group[i]
  sig_range[[i]] <- unique(ratings$timespan[ratings$sig_id == interest_group[i]])
}
names(sig_range) <- interest_group

# get rid of redundant ratings
rows <- list()
for (i in 1:length(interest_group)){
  if ("2019-2020" %in% sig_range[[i]]){
    rows[[i]] <- which(ratings$sig_id == interest_group[[i]] & ratings$timespan != "2019-2020")
  }
}
ratings <- ratings[-unlist(rows),]

# determine time range for each SIG
sig_range <- list()
for (i in 1:length(interest_group)){
  sig_range[[i]] <- interest_group[i]
  sig_range[[i]] <- unique(ratings$timespan[ratings$sig_id == interest_group[i]])
}
names(sig_range) <- interest_group
sig_range

# SIGs with multiple ratings in relevant period
multiples <- c()
for (i in 1:length(sig_range)){
  if (length(sig_range[[i]]) > 1){multiples[i] <- names(sig_range)[i]}
}
multiples <- multiples[!is.na(multiples)]
multiples

# average for SIGs w/multiple ratings
temp <- ratings[which(ratings$sig_id %in% multiples),]
temp <- aggregate(temp, 
                  by=list(temp$candidate_id, temp$sig_id), 
                  mean, na.rm = T)
ratings$rating_avg <- ratings$rating
for (j in 1:length(multiples)){
  for (i in 1:nrow(ratings)){
    if (ratings[i,"sig_id"] == multiples[j]){
      ratings$rating_avg[i] <- temp$rating[temp$Group.1 == ratings$candidate_id[i] & temp$Group.2 == multiples[j]] 
    }
  }
}

# how many/which available interest groups?
unique(ratings$sig_id)

# export
write.csv(ratings, "Data/Vote Smart Ratings_All.csv", row.names = F)







